from django.http import HttpResponse
from django.contrib.auth import authenticate
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from allocation.models import * 
from django.forms import modelformset_factory, formset_factory, modelform_factory, ModelForm
from django.views import generic
from django.views.generic.edit import *
from django.shortcuts import render, redirect, get_object_or_404
from django.conf import settings
from django.core.files.storage import FileSystemStorage
from django.contrib.auth import authenticate, login
from django import forms
import csv
from django.urls import reverse_lazy
from allocation.forms import *
from alloc import *

def signup(request):
    if request.method == 'POST':
        form = InstiForm(request.POST)
        if form.is_valid():
            user = form.save()
            user.is_staff=True
            user.save()
            new_insti=Institute(name=user.username)
            new_insti.save()
            return HttpResponseRedirect('/allocation/admin')
    else:
        form = InstiForm()
    return render(request, 'registration/register.html', {'form': form})

def button_action(request):
    institute=request.user.username
    allocator(institute)
    return HttpResponse("success")

def home(request):
    documents = Document.objects.all()
    return render(request, 'core/home.html', { 'documents': documents })
    return HttpResponse("On home")

def choice_make(request):
    if request.method == "POST":
        csvfile = request.FILES['myfile']
        file_data = csvfile.read().decode("utf-8")
        lines = file_data.split("\n")
        user_name = request.user.username
        institute = get_object_or_404(Institute, name=user_name)
        for line in lines:
            line =  line.split(',')
            tmp = institute.choices.create(choice_name=line[0],capacity=line[1])
            tmp.save()
        return HttpResponse("successful")
    return HttpResponse("Error")


def applicant_make(request):
    if request.method == "POST":
        csvfile = request.FILES['myfile']
        file_data = csvfile.read().decode("utf-8")
        user_name = request.user.username
        institute = get_object_or_404(Institute, name=user_name)
        lines = file_data.split("\n")
        for line in lines:
            line = line.split(',')
            tmp = institute.applicants.create(name = line[0],rank = line[1])
            tmp.save()
            user = User.objects.create_user(username=line[2],email=line[3],password=line[4])
            user.is_staff = True
            user.save()
        return HttpResponse("successful")
    return HttpResponse("Error")

## Makes the "fill choices" for the student login
# @param applicant The applicant whose form has to be displayed
# @return returns the form for the corresponding applicant
# This view requires login to be displayed. 
def make_application_form(applicant1):
    # The inbuilt Model form class is used for the form
    class ApplicationForm(forms.ModelForm):
        choice = forms.ModelChoiceField(queryset=Choice.objects.filter(institute=applicant1.institute))#< The queryset is the choices available in the dropdown in the form.
        applicant = forms.ModelChoiceField(widget=forms.HiddenInput(), initial=applicant1, queryset=Applicant.objects.all() )#<this field is hidden since an applicant can apply for himself only.
        class Meta:
            model = Application
            fields = ['applicant','choice', 'priority'] #< These are the fields to be displayed
    return ApplicationForm

## View for applicant login
# @param rerquest
# This view requires login to be displayed. 
@login_required(login_url='/admin/login')
def index(request):
    applicant=get_object_or_404(Applicant,name=request.user.username) #< If an applicant exists with the username of the account, this returns the applicant. Else it returns 404 error.
    ApplicationForm = make_application_form(applicant)
    ApplicationFormSet = modelformset_factory(Application, form=ApplicationForm, extra=5) # used inbuilt function to create a formset from a form.
    if request.method == "POST":
        formset = ApplicationFormSet(request.POST, queryset=Application.objects.filter(applicant=applicant))
        if formset.is_valid():#< checks if the given information is valid.
            #< if the formset is valid, then save the form and redirect to the same page giving success as the context
            formset.save()
            return render(request, 'allocation/index.html', {'applicant': applicant,  'formset': formset, 'success':True,})
        else:
            #< if the formset is invalid, then save the form and redirect to the same page giving errors as the context
            form_errors = formset.errors
            return render(request, 'allocation/index.html', {'applicant': applicant,  'formset': formset,'form_errors': form_errors})
    else:
        formset = ApplicationFormSet(queryset=Application.objects.filter(applicant=applicant),)
        return render(request, 'allocation/index.html', {'applicant': applicant,  'formset': formset,})

## View for institute login
# @param rerquest
# This view requires login to be displayed. 
@login_required(login_url='/admin/login')
def admin1(request):
	institute=get_object_or_404(Institute,name=request.user.username)#< If an institute exists with the username of the account, this returns the institute. Else it returns 404 error.
	return render(request, 'allocation/admin1.html',{'institute': institute})

## View for institute page which lists all the choices of the institute 
# This is a generic list view provided by django to easily create views of models.
# The page also includes button for uploading csv file for creation of multiple choices at once
# The corresponding html file is choice_list.html
class ChoiceListView(generic.ListView):
	model = Choice #< specifies the model for which the generic view is to be created
    ##Context
    # Used to pass an additional context in the html file
	def get_context_data(self, **kwargs):
		institute=get_object_or_404(Institute,name=self.request.user.username)
		context = super(ChoiceListView, self).get_context_data(**kwargs)
		context['institute'] = institute #< Because of this line we can access the institute information through the variable named 'institute' in the html file
		return context
    ##Queryset
    # Used to specify the list of choices which are to be displayed
	def get_queryset(self):
		institute=get_object_or_404(Institute,name=self.request.user.username)
		return Choice.objects.filter(institute__name=institute.name) #< this returns the list of choices corresponding to the institute which is logged in currently

## View for institute page which lists all the applicants of the institute 
# This is a generic list view provided by django to easily create views of models.
# The page also includes button for uploading csv file for creation of multiple applicants at once
# The corresponding html file is applicants_list.html
class ApplicantListView(generic.ListView):
    model = Applicant#< specifies the model for which the generic view is to be created
    ##Context
    # Used to pass an additional context in the html file
    def get_context_data(self, **kwargs):
        institute=get_object_or_404(Institute,name=self.request.user.username)
        context = super(ApplicantListView, self).get_context_data(**kwargs)
        context['institute'] = institute #< Because of this line we can access the institute information through the variable named 'institute' in the html file
        return context
    ##Queryset
    # Used to specify the list of applicants which are to be displayed
    def get_queryset(self):
        institute=get_object_or_404(Institute,name=self.request.user.username)
        return Applicant.objects.filter(institute__name=institute.name) #< this returns the list of choices corresponding to the institute which is logged in currently

## View for institute page which displays a particular choice of the institute in detail 
# This is a generic detail view provided by django to easily create detail views of models.
# The page also includes update choice and delete choice buttons
# The corresponding html file is choice_detail.html
class ChoiceDetailView(generic.DetailView):
    model = Choice #< specifies the model for which the generic view is to be created
    ##Context
    # Used to pass an additional context in the html file
    def get_context_data(self, **kwargs):
        institute=get_object_or_404(Institute,name=self.request.user.username)
        context = super(ChoiceDetailView, self).get_context_data(**kwargs)
        context['institute'] = institute #< Because of this line we can access the institute information through the variable named 'institute' in the html file
        return context

## View for institute page which displays profile of a particular applicant of the institute in detail 
# This is a generic detail view provided by django to easily detail create views of models.
# The page also includes update applicant and delete applicant buttons
# The corresponding html file is applicant_detail.html
class ApplicantDetailView(generic.DetailView):
    model = Applicant #< specifies the model for which the generic view is to be created
    ##Context
    # Used to pass an additional context in the html file
    def get_context_data(self, **kwargs):
        institute=get_object_or_404(Institute,name=self.request.user.username)
        context = super(ApplicantDetailView, self).get_context_data(**kwargs)
        context['institute'] = institute #< Because of this line we can access the institute information through the variable named 'institute' in the html file
        return context

## View for institute page which displays a form to update the details of choices
# This is a generic update view provided by django to easily create update views of models.
# The corresponding html file is choice_update.html
class ChoiceUpdate(UpdateView):
    model = Choice #< specifies the model for which the generic view is to be created
    fields = ['choice_name','capacity'] #< specifies the fields that will be displayed on the rendered form
    template_name_suffix = '_update' # Specifies that the html file is choice_update.html
    success_url = reverse_lazy('allocation:choice_list') #< this is the redirect URL upon successful updation of choice

## View for institute page which displays a form to delete the choices
# This is a generic delete view provided by django to easily create delete views of models.
# The html file for delete confirmation is choice_confirm_delete.html
class ChoiceDelete(DeleteView):
    model = Choice #< specifies the model for which the generic view is to be created
    success_url = reverse_lazy('allocation:choice_list') #< this is the redirect URL upon successful deletion of choice

## View for institute page which displays a form to update the details of applicants
# This is a generic update view provided by django to easily create update views of models.
# The corresponding html file is applicant_update.html
class ApplicantUpdate(UpdateView):
    model = Applicant #< specifies the model for which the generic view is to be created
    fields = ['name','rank']#< specifies the fields that will be displayed on the rendered form
    template_name_suffix = '_update' # Specifies that the html file is choice_update.html
    success_url = reverse_lazy('allocation:applicant_list')#< this is the redirect URL upon successful updation of applicant

## View for institute page which displays a form to delete the applicants
# This is a generic delete view provided by django to easily create delete views of models.
# The html file for delete confirmation is applicant_confirm_delete.html
class ApplicantDelete(DeleteView):
    model = Applicant #< specifies the model for which the generic view is to be created
    success_url = reverse_lazy('allocation:applicant_list')#< this is the redirect URL upon successful deletion of applicant

def freeze(request):
    if request.method == "POST":
        applicant=Applicant.objects.get(pk=request.POST["applicant"])
        applicant.is_float=False
        applicant.save()
        return HttpResponseRedirect('/allocation')
    else:
        return HttpResponseRedirect('/allocation')

def drop(request):
    if request.method == "POST":
        applicant=Applicant.objects.get(pk=request.POST["applicant"])
        applicant.is_float=False
        given_choice=applicant.alloted_choice.all()
        if given_choice:
            g=given_choice[0]
            g.seats_filled=g.seats_filled-1
            applicant.alloted_choice.clear()
            g.save()
        applicant.save()
        return HttpResponseRedirect('/allocation')
    else:
        return HttpResponseRedirect('/allocation')

def float(request):
    return HttpResponseRedirect('/allocation')